@@ -49,8 +49,7 @@ void CmdExtract::DoExtract()
       if (Code!=EXTRACT_ARC_REPEAT)
         break;
     }
-    if (FindFile::FastFind(ArcName,&FD))
-      DataIO.ProcessedArcSize+=FD.Size;
+    DataIO.ProcessedArcSize+=DataIO.LastArcSize;
   }
 
   // Clean user entered password. Not really required, just for extra safety.
@@ -82,7 +81,7 @@ void CmdExtract::DoExtract()
 
 void CmdExtract::ExtractArchiveInit(Archive &Arc)
 {
-  DataIO.UnpArcSize=Arc.FileLength();
+  DataIO.AdjustTotalArcSize(&Arc);
 
   FileCount=0;
   MatchedArgs=0;
@@ -105,8 +104,22 @@ void CmdExtract::ExtractArchiveInit(Archive &Arc)
 EXTRACT_ARC_CODE CmdExtract::ExtractArchive()
 {
   Archive Arc(Cmd);
-  if (!Arc.WOpen(ArcName))
-    return EXTRACT_ARC_NEXT;
+  if (*Cmd->UseStdin!=0)
+  {
+    Arc.SetHandleType(FILE_HANDLESTD);
+#ifdef USE_QOPEN
+    Arc.SetProhibitQOpen(true);
+#endif
+  }
+  else
+  {
+#if defined(_WIN_ALL) && !defined(SFX_MODULE) // WinRAR GUI code also resets the cache.
+    if (*Cmd->Command=='T' || Cmd->Test)
+      ResetFileCache(ArcName); // Reset the file cache when testing an archive.
+#endif
+    if (!Arc.WOpen(ArcName))
+      return EXTRACT_ARC_NEXT;
+  }
 
   if (!Arc.IsArchive(true))
   {
@@ -216,14 +229,11 @@ EXTRACT_ARC_CODE CmdExtract::ExtractArchive()
       if (Repeat)
       {
         // If we started extraction from not first volume and need to
-        // restart it from first, we must correct DataIO.TotalArcSize
-        // for correct total progress display. We subtract the size
-        // of current volume and all volumes after it and add the size
-        // of new (first) volume.
-        FindData OldArc,NewArc;
-        if (FindFile::FastFind(Arc.FileName,&OldArc) &&
-            FindFile::FastFind(ArcName,&NewArc))
-          DataIO.TotalArcSize-=VolumeSetSize+OldArc.Size-NewArc.Size;
+        // restart it from first, we must set DataIO.TotalArcSize to size
+        // of new first volume to display the total progress correctly.
+        FindData NewArc;
+        if (FindFile::FastFind(ArcName,&NewArc))
+          DataIO.TotalArcSize=NewArc.Size;
         return EXTRACT_ARC_REPEAT;
       }
       else
@@ -403,6 +413,12 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
   FirstFile=false;
 #endif
 
+  if (Arc.FileHead.Encrypted && Cmd->SkipEncrypted)
+    if (Arc.Solid)
+      return false; // Abort the entire extraction for solid archive.
+    else
+      MatchFound=false; // Skip only the current file for non-solid archive.
+  
   if (MatchFound || (SkipSolid=Arc.Solid)!=0)
   {
     // First common call of uiStartFileExtract. It is done before overwrite
@@ -632,7 +648,7 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
 
       uint64 Preallocated=0;
       if (!TestMode && !Arc.BrokenHeader && Arc.FileHead.UnpSize>1000000 &&
-          Arc.FileHead.PackSize*1024>Arc.FileHead.UnpSize &&
+          Arc.FileHead.PackSize*1024>Arc.FileHead.UnpSize && Arc.IsSeekable() &&
           (Arc.FileHead.UnpSize<100000000 || Arc.FileLength()>Arc.FileHead.PackSize))
       {
         CurFile.Prealloc(Arc.FileHead.UnpSize);
@@ -650,11 +666,14 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
 
         if (Type==FSREDIR_HARDLINK || Type==FSREDIR_FILECOPY)
         {
+          wchar RedirName[NM];
+          ConvertPath(Arc.FileHead.RedirName,RedirName,ASIZE(RedirName));
+
           wchar NameExisting[NM];
-          ExtrPrepareName(Arc,Arc.FileHead.RedirName,NameExisting,ASIZE(NameExisting));
+          ExtrPrepareName(Arc,RedirName,NameExisting,ASIZE(NameExisting));
           if (FileCreateMode && *NameExisting!=0) // *NameExisting can be 0 in case of excessive -ap switch.
             if (Type==FSREDIR_HARDLINK)
-              LinkSuccess=ExtractHardlink(DestFileName,NameExisting,ASIZE(NameExisting));
+              LinkSuccess=ExtractHardlink(Cmd,DestFileName,NameExisting,ASIZE(NameExisting));
             else
               LinkSuccess=ExtractFileCopy(CurFile,Arc.FileName,DestFileName,NameExisting,ASIZE(NameExisting));
         }
@@ -666,7 +685,7 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
           }
           else
           {
-            uiMsg(UIERROR_UNKNOWNEXTRA, Arc.FileName, DestFileName);
+            uiMsg(UIERROR_UNKNOWNEXTRA,Arc.FileName,DestFileName);
             LinkSuccess=false;
           }
           
@@ -747,7 +766,13 @@ bool CmdExtract::ExtractCurrentFile(Archive &Arc,size_t HeaderSize,bool &Repeat)
         }
       }
       else
-        mprintf(L"\b\b\b\b\b     ");
+      {
+        // We check SkipSolid to remove percent for skipped solid files only.
+        // We must not apply these \b to links with ShowChecksum==false
+        // and their possible error messages.
+        if (SkipSolid) 
+          mprintf(L"\b\b\b\b\b     ");
+      }
 
       // If we successfully unpacked a hard link, we wish to set its file
       // attributes. Hard link shares file metadata with link target,
@@ -882,6 +907,8 @@ void CmdExtract::ExtrPrepareName(Archive &Arc,const wchar *ArcFileName,wchar *De
      wchar LastChar=*PointToLastChar(Cmd->ExtrPath);
     // We need IsPathDiv check here to correctly handle Unix forward slash
     // in the end of destination path in Windows: rar x arc dest/
+    // so we call IsPathDiv first instead of just calling AddEndSlash,
+    // which checks for only one type of path separator.
     // IsDriveDiv is needed for current drive dir: rar x arc d:
     if (!IsPathDiv(LastChar) && !IsDriveDiv(LastChar))
     {
@@ -913,19 +940,13 @@ void CmdExtract::ExtrPrepareName(Archive &Arc,const wchar *ArcFileName,wchar *De
 #endif
 
 #ifndef SFX_MODULE
-  size_t ArcPathLength=wcslen(Cmd->ArcPath);
+  wchar *ArcPath=*Cmd->ExclArcPath!=0 ? Cmd->ExclArcPath:Cmd->ArcPath;
+  size_t ArcPathLength=wcslen(ArcPath);
   if (ArcPathLength>0)
   {
     size_t NameLength=wcslen(ArcFileName);
-
-    // Earlier we compared lengths only here, but then noticed a cosmetic bug
-    // in WinRAR. When extracting a file reference from subfolder with
-    // "Extract relative paths", so WinRAR sets ArcPath, if reference target
-    // is missing, error message removed ArcPath both from reference and target
-    // names. If target was stored in another folder, its name looked wrong.
-    if (NameLength>=ArcPathLength && 
-        wcsnicompc(Cmd->ArcPath,ArcFileName,ArcPathLength)==0 &&
-        (IsPathDiv(Cmd->ArcPath[ArcPathLength-1]) || 
+    if (NameLength>=ArcPathLength &&  wcsnicompc(ArcPath,ArcFileName,ArcPathLength)==0 &&
+        (IsPathDiv(ArcPath[ArcPathLength-1]) || 
          IsPathDiv(ArcFileName[ArcPathLength]) || ArcFileName[ArcPathLength]==0))
     {
       ArcFileName+=Min(ArcPathLength,NameLength);
@@ -958,7 +979,7 @@ void CmdExtract::ExtrPrepareName(Archive &Arc,const wchar *ArcFileName,wchar *De
   // Must do after Cmd->ArcPath processing above, so file name and arc path
   // trailing spaces are in sync.
   if (!Cmd->AllowIncompatNames)
-    MakeNameCompatible(DestName);
+    MakeNameCompatible(DestName,DestSize);
 #endif
 
   wchar DiskLetter=toupperw(DestName[0]);
@@ -1019,11 +1040,7 @@ bool CmdExtract::ExtrGetPassword(Archive &Arc,const wchar *ArcFileName)
     if (!uiGetPassword(UIPASSWORD_FILE,ArcFileName,&Cmd->Password)/* || !Cmd->Password.IsSet()*/)
     {
       // Suppress "test is ok" message if user cancelled the password prompt.
-// 2019.03.23: If some archives are tested ok and prompt is cancelled for others,
-// do we really need to suppress "test is ok"? Also if we set an empty password
-// and "Use for all archives" in WinRAR Ctrl+P and skip some encrypted archives.
-// We commented out this UIERROR_INCERRCOUNT for now.
-//      uiMsg(UIERROR_INCERRCOUNT);
+      uiMsg(UIERROR_INCERRCOUNT);
       return false;
     }
     Cmd->ManualPassword=true;
@@ -1100,19 +1117,23 @@ void CmdExtract::ExtrCreateDir(Archive &Arc,const wchar *ArcFileName)
     }
     if (!DirExist)
     {
-      CreatePath(DestFileName,true);
+      CreatePath(DestFileName,true,Cmd->DisableNames);
       MDCode=MakeDir(DestFileName,!Cmd->IgnoreGeneralAttr,Arc.FileHead.FileAttr);
-      if (MDCode!=MKDIR_SUCCESS)
+      if (MDCode!=MKDIR_SUCCESS && !IsNameUsable(DestFileName))
       {
+        uiMsg(UIMSG_CORRECTINGNAME,Arc.FileName);
         wchar OrigName[ASIZE(DestFileName)];
         wcsncpyz(OrigName,DestFileName,ASIZE(OrigName));
         MakeNameUsable(DestFileName,true);
-        CreatePath(DestFileName,true);
-        MDCode=MakeDir(DestFileName,!Cmd->IgnoreGeneralAttr,Arc.FileHead.FileAttr);
 #ifndef SFX_MODULE
-        if (MDCode==MKDIR_SUCCESS)
-          uiMsg(UIERROR_RENAMING,Arc.FileName,OrigName,DestFileName);
+        uiMsg(UIERROR_RENAMING,Arc.FileName,OrigName,DestFileName);
 #endif
+        DirExist=FileExist(DestFileName) && IsDir(GetFileAttr(DestFileName));
+        if (!DirExist)
+        {
+          CreatePath(DestFileName,true,Cmd->DisableNames);
+          MDCode=MakeDir(DestFileName,!Cmd->IgnoreGeneralAttr,Arc.FileHead.FileAttr);
+        }
       }
     }
   }
@@ -1191,7 +1212,7 @@ bool CmdExtract::ExtrCreateFile(Archive &Arc,File &CurFile)
 
           MakeNameUsable(DestFileName,true);
 
-          CreatePath(DestFileName,true);
+          CreatePath(DestFileName,true,Cmd->DisableNames);
           if (FileCreate(Cmd,&CurFile,DestFileName,ASIZE(DestFileName),&UserReject,Arc.FileHead.UnpSize,&Arc.FileHead.mtime,true))
           {
 #ifndef SFX_MODULE
