@@ -29,12 +29,6 @@ MKDIR_CODE MakeDir(const wchar *Name,bool SetAttr,uint Attr)
   WideToChar(Name,NameA,ASIZE(NameA));
   mode_t uattr=SetAttr ? (mode_t)Attr:0777;
   int ErrCode=mkdir(NameA,uattr);
-#ifdef _ANDROID
-  if (ErrCode==-1 && errno!=ENOENT)
-    ErrCode=JniMkdir(Name) ? 0 : -1;  // If external card is read-only for usual file API.
-  if (ErrCode!=-1)
-    JniFileNotify(Name,false);
-#endif
   if (ErrCode==-1)
     return errno==ENOENT ? MKDIR_BADPATH:MKDIR_ERROR;
   return MKDIR_SUCCESS;
@@ -44,7 +38,7 @@ MKDIR_CODE MakeDir(const wchar *Name,bool SetAttr,uint Attr)
 }
 
 
-bool CreatePath(const wchar *Path,bool SkipLastName)
+bool CreatePath(const wchar *Path,bool SkipLastName,bool Silent)
 {
   if (Path==NULL || *Path==0)
     return false;
@@ -79,13 +73,11 @@ bool CreatePath(const wchar *Path,bool SkipLastName)
       DirName[s-Path]=0;
 
       Success=MakeDir(DirName,true,DirAttr)==MKDIR_SUCCESS;
-#ifndef GUI
-      if (Success)
+      if (Success && !Silent)
       {
         mprintf(St(MCreatDir),DirName);
         mprintf(L" %s",St(MOk));
       }
-#endif
     }
   }
   if (!SkipLastName && !IsPathDiv(*PointToLastChar(Path)))
@@ -96,7 +88,7 @@ bool CreatePath(const wchar *Path,bool SkipLastName)
 
 void SetDirTime(const wchar *Name,RarTime *ftm,RarTime *ftc,RarTime *fta)
 {
-#ifdef _WIN_ALL
+#if defined(_WIN_ALL)
   bool sm=ftm!=NULL && ftm->IsSet();
   bool sc=ftc!=NULL && ftc->IsSet();
   bool sa=fta!=NULL && fta->IsSet();
@@ -120,11 +112,11 @@ void SetDirTime(const wchar *Name,RarTime *ftm,RarTime *ftc,RarTime *fta)
     return;
   FILETIME fm,fc,fa;
   if (sm)
-    ftm->GetWin32(&fm);
+    ftm->GetWinFT(&fm);
   if (sc)
-    ftc->GetWin32(&fc);
+    ftc->GetWinFT(&fc);
   if (sa)
-    fta->GetWin32(&fa);
+    fta->GetWinFT(&fa);
   SetFileTime(hFile,sc ? &fc:NULL,sa ? &fa:NULL,sm ? &fm:NULL);
   CloseHandle(hFile);
   if (ResetAttr)
@@ -138,7 +130,7 @@ void SetDirTime(const wchar *Name,RarTime *ftm,RarTime *ftc,RarTime *fta)
 
 bool IsRemovable(const wchar *Name)
 {
-#ifdef _WIN_ALL
+#if defined(_WIN_ALL)
   wchar Root[NM];
   GetPathRoot(Name,Root,ASIZE(Root));
   int Type=GetDriveType(*Root!=0 ? Root:NULL);
@@ -328,19 +320,52 @@ bool SetFileAttr(const wchar *Name,uint Attr)
 }
 
 
+#if 0
+wchar *MkTemp(wchar *Name,size_t MaxSize)
+{
+  size_t Length=wcslen(Name);
+
+  RarTime CurTime;
+  CurTime.SetCurrentTime();
+
+  // We cannot use CurTime.GetWin() as is, because its lowest bits can
+  // have low informational value, like being a zero or few fixed numbers.
+  uint Random=(uint)(CurTime.GetWin()/100000);
+
+  // Using PID we guarantee that different RAR copies use different temp names
+  // even if started in exactly the same time.
+  uint PID=0;
+#ifdef _WIN_ALL
+  PID=(uint)GetCurrentProcessId();
+#elif defined(_UNIX)
+  PID=(uint)getpid();
+#endif
+
+  for (uint Attempt=0;;Attempt++)
+  {
+    uint Ext=Random%50000+Attempt;
+    wchar RndText[50];
+    swprintf(RndText,ASIZE(RndText),L"%u.%03u",PID,Ext);
+    if (Length+wcslen(RndText)>=MaxSize || Attempt==1000)
+      return NULL;
+    wcsncpyz(Name+Length,RndText,MaxSize-Length);
+    if (!FileExist(Name))
+      break;
+  }
+  return Name;
+}
+#endif
 
 
-#if !defined(SFX_MODULE) && !defined(SHELL_EXT) && !defined(SETUP)
+#if !defined(SFX_MODULE)
 void CalcFileSum(File *SrcFile,uint *CRC32,byte *Blake2,uint Threads,int64 Size,uint Flags)
 {
-  SaveFilePos SavePos(*SrcFile);
+  int64 SavePos=SrcFile->Tell();
 #ifndef SILENT
   int64 FileLength=Size==INT64NDF ? SrcFile->FileLength() : Size;
 #endif
 
-#ifndef GUI
   if ((Flags & (CALCFSUM_SHOWTEXT|CALCFSUM_SHOWPERCENT))!=0)
-#endif
     uiMsg(UIEVENT_FILESUMSTART);
 
   if ((Flags & CALCFSUM_CURPOS)==0)
@@ -372,12 +397,14 @@ void CalcFileSum(File *SrcFile,uint *CRC32,byte *Blake2,uint Threads,int64 Size,
     {
 #ifndef SILENT
       if ((Flags & CALCFSUM_SHOWPROGRESS)!=0)
-        uiExtractProgress(TotalRead,FileLength,TotalRead,FileLength);
+      {
+        // Update only the current file progress in WinRAR, set the total to 0
+        // to keep it as is. It looks better for WinRAR,
+        uiExtractProgress(TotalRead,FileLength,0,0);
+      }
       else
       {
-#ifndef GUI
         if ((Flags & CALCFSUM_SHOWPERCENT)!=0)
-#endif
           uiMsg(UIEVENT_FILESUMPROGRESS,ToPercent(TotalRead,FileLength));
       }
 #endif
@@ -392,9 +419,9 @@ void CalcFileSum(File *SrcFile,uint *CRC32,byte *Blake2,uint Threads,int64 Size,
     if (Size!=INT64NDF)
       Size-=ReadSize;
   }
-#ifndef GUI
+  SrcFile->Seek(SavePos,SEEK_SET);
+
   if ((Flags & CALCFSUM_SHOWPERCENT)!=0)
-#endif
     uiMsg(UIEVENT_FILESUMEND);
 
   if (CRC32!=NULL)
@@ -426,15 +453,6 @@ bool RenameFile(const wchar *SrcName,const wchar *DestName)
   WideToChar(SrcName,SrcNameA,ASIZE(SrcNameA));
   WideToChar(DestName,DestNameA,ASIZE(DestNameA));
   bool Success=rename(SrcNameA,DestNameA)==0;
-#ifdef _ANDROID
-  if (!Success)
-    Success=JniRename(SrcName,DestName); // If external card is read-only for usual file API.
-  if (Success)
-  {
-    JniFileNotify(SrcName,true);
-    JniFileNotify(DestName,false);
-  }
-#endif
   return Success;
 #endif
 }
@@ -455,17 +473,29 @@ bool DelFile(const wchar *Name)
   char NameA[NM];
   WideToChar(Name,NameA,ASIZE(NameA));
   bool Success=remove(NameA)==0;
-#ifdef _ANDROID
-  if (!Success)
-    Success=JniDelete(Name);
-  if (Success)
-    JniFileNotify(Name,true);
-#endif
   return Success;
 #endif
 }
 
 
+bool DelDir(const wchar *Name)
+{
+#ifdef _WIN_ALL
+  bool Success=RemoveDirectory(Name)!=0;
+  if (!Success)
+  {
+    wchar LongName[NM];
+    if (GetWinLongPath(Name,LongName,ASIZE(LongName)))
+      Success=RemoveDirectory(LongName)!=0;
+  }
+  return Success;
+#else
+  char NameA[NM];
+  WideToChar(Name,NameA,ASIZE(NameA));
+  bool Success=rmdir(NameA)==0;
+  return Success;
+#endif
+}
 
 
 #if defined(_WIN_ALL) && !defined(SFX_MODULE)
@@ -491,6 +521,18 @@ bool SetFileCompression(const wchar *Name,bool State)
   CloseHandle(hFile);
   return RetCode!=0;
 }
+
+
+void ResetFileCache(const wchar *Name)
+{
+  // To reset file cache in Windows it is enough to open it with
+  // FILE_FLAG_NO_BUFFERING and then close it.
+  HANDLE hSrc=CreateFile(Name,GENERIC_READ,
+                         FILE_SHARE_READ|FILE_SHARE_WRITE,
+                         NULL,OPEN_EXISTING,FILE_FLAG_NO_BUFFERING,NULL);
+  if (hSrc!=INVALID_HANDLE_VALUE)
+    CloseHandle(hSrc);
+}
 #endif
 
 
