@@ -72,11 +72,11 @@
 #define   MCHelpSwDW         L"\n  dw            Wipe files after archiving"
 #define   MCHelpSwEa         L"\n  e[+]<attr>    Set file exclude and include attributes"
 #define   MCHelpSwED         L"\n  ed            Do not add empty directories"
-#define   MCHelpSwEN         L"\n  en            Do not put 'end of archive' block"
 #define   MCHelpSwEP         L"\n  ep            Exclude paths from names"
 #define   MCHelpSwEP1        L"\n  ep1           Exclude base directory from names"
 #define   MCHelpSwEP2        L"\n  ep2           Expand paths to full"
 #define   MCHelpSwEP3        L"\n  ep3           Expand paths to full including the drive letter"
+#define   MCHelpSwEP4        L"\n  ep4<path>     Exclude the path prefix from names"
 #define   MCHelpSwF          L"\n  f             Freshen files"
 #define   MCHelpSwHP         L"\n  hp[password]  Encrypt both file data and headers"
 #define   MCHelpSwHT         L"\n  ht[b|c]       Select hash type [BLAKE2,CRC32] for file checksum"
@@ -95,6 +95,7 @@
 #define   MCHelpSwMA         L"\n  ma[4|5]       Specify a version of archiving format"
 #define   MCHelpSwMC         L"\n  mc<par>       Set advanced compression parameters"
 #define   MCHelpSwMD         L"\n  md<n>[k,m,g]  Dictionary size in KB, MB or GB"
+#define   MCHelpSwME         L"\n  me[par]       Set encryption parameters"
 #define   MCHelpSwMS         L"\n  ms[ext;ext]   Specify file types to store"
 #define   MCHelpSwMT         L"\n  mt<threads>   Set the number of threads"
 #define   MCHelpSwN          L"\n  n<file>       Additionally filter included files"
@@ -106,11 +107,11 @@
 #define   MCHelpSwOI         L"\n  oi[0-4][:min] Save identical files as references"
 #define   MCHelpSwOL         L"\n  ol[a]         Process symbolic links as the link [absolute paths]"
 #define   MCHelpSwONI        L"\n  oni           Allow potentially incompatible names"
+#define   MCHelpSwOP         L"\n  op<path>      Set the output path for extracted files"
 #define   MCHelpSwOR         L"\n  or            Rename files automatically"
 #define   MCHelpSwOS         L"\n  os            Save NTFS streams"
 #define   MCHelpSwOW         L"\n  ow            Save or restore file owner and group"
 #define   MCHelpSwP          L"\n  p[password]   Set password"
-#define   MCHelpSwPm         L"\n  p-            Do not query password"
 #define   MCHelpSwQO         L"\n  qo[-|+]       Add quick open information [none|force]"
 #define   MCHelpSwR          L"\n  r             Recurse subdirectories"
 #define   MCHelpSwRm         L"\n  r-            Disable recursion"
@@ -272,6 +273,9 @@
 #define   MListMtime         L"mtime"
 #define   MListCtime         L"ctime"
 #define   MListAtime         L"atime"
+#define   MListModified      L"Modified"
+#define   MListCreated       L"Created"
+#define   MListAccessed      L"Accessed"
 #define   MListAttr          L"Attributes"
 #define   MListFlags         L"Flags"
 #define   MListCompInfo      L"Compression"
@@ -321,7 +325,7 @@
 #define   MStreamUnknown     L"\nWARNING: Unknown format of %s stream data\n"
 #define   MInvalidName       L"\nERROR: Invalid file name %s"
 #define   MProcessArc        L"\n\nProcessing archive %s"
-#define   MCorrectingName    L"\nWARNING: Attempting to correct the invalid file name"
+#define   MCorrectingName    L"\nWARNING: Attempting to correct the invalid file or directory name"
 #define   MUnpCannotMerge    L"\nWARNING: You need to start extraction from a previous volume to unpack %s"
 #define   MUnknownOption     L"\nERROR: Unknown option: %s"
 #define   MSubHeadCorrupt    L"\nERROR: Corrupt data header found, ignored"
@@ -385,3 +389,8 @@
 #define   MErrReadTrunc      L"\n%s is archived incompletely because of read error.\n"
 #define   MErrReadCount      L"\n%u files are archived incompletely because of read errors."
 #define   MDirNameExists     L"\nDirectory with such name already exists"
+#define   MStdinNoInput      L"\nKeyboard input is not allowed when reading data from stdin"
+#define   MTruncPsw          L"\nPassword exceeds the maximum allowed length of %u characters and will be truncated."
+#define   MAdjustValue       L"\nAdjusting %s value to %s."
+#define   MOpFailed          L"\nOperation failed"
+#define   MSkipEncArc        L"\nSkipping the encrypted archive %s"
