@@ -32,11 +32,15 @@
 #define STRICT 1
 #endif
 
+// 'ifndef' check here is needed for unrar.dll header to avoid macro
+// re-definition warnings in third party projects.
+#ifndef UNICODE
 #define UNICODE
-#undef WINVER
-#undef _WIN32_WINNT
-#define WINVER 0x0501
-#define _WIN32_WINNT 0x0501
+#define _UNICODE // Set _T() macro to convert from narrow to wide strings.
+#endif
+
+#define WINVER _WIN32_WINNT_WINXP
+#define _WIN32_WINNT _WIN32_WINNT_WINXP
 
 #if !defined(ZIPSFX)
 #define RAR_SMP
@@ -74,8 +78,11 @@
   #include <direct.h>
   #include <intrin.h>
 
-  #define USE_SSE
-  #define SSE_ALIGNMENT 16
+  // Use SSE only for x86/x64, not ARM Windows.
+  #if defined(_M_IX86) || defined(_M_X64)
+    #define USE_SSE
+    #define SSE_ALIGNMENT 16
+  #endif
 #else
   #include <dirent.h>
 #endif // _MSC_VER
@@ -128,7 +135,7 @@
 
 #ifdef _UNIX
 
-#define  NM  2048
+#define NM  2048
 
 #include <unistd.h>
 #include <sys/types.h>
@@ -207,7 +214,8 @@
   #endif
 #endif
 
-#if _POSIX_C_SOURCE >= 200809L
+// Unlike Apple x64, utimensat shall be available in all Apple M1 systems.
+#if _POSIX_C_SOURCE >= 200809L || defined(__APPLE__) && defined(__arm64__)
   #define UNIX_TIME_NS // Nanosecond time precision in Unix.
 #endif
 
