@@ -28,7 +28,8 @@ void IntToExt(const char *Src,char *Dest,size_t DestSize)
 }
 
 
-// Convert archived names to Unicode. Allow user to select a code page in GUI.
+// Convert archived names and comments to Unicode.
+// Allows user to select a code page in GUI.
 void ArcCharToWide(const char *Src,wchar *Dest,size_t DestSize,ACTW_ENCODING Encoding)
 {
 #if defined(_WIN_ALL) // Console Windows RAR.
@@ -36,11 +37,12 @@ void ArcCharToWide(const char *Src,wchar *Dest,size_t DestSize,ACTW_ENCODING Enc
     UtfToWide(Src,Dest,DestSize);
   else
   {
-    char NameA[NM];
+    Array<char> NameA;
     if (Encoding==ACTW_OEM)
     {
-      IntToExt(Src,NameA,ASIZE(NameA));
-      Src=NameA;
+      NameA.Alloc(DestSize+1);
+      IntToExt(Src,&NameA[0],NameA.Size());
+      Src=&NameA[0];
     }
     CharToWide(Src,Dest,DestSize);
   }
@@ -58,6 +60,8 @@ void ArcCharToWide(const char *Src,wchar *Dest,size_t DestSize,ACTW_ENCODING Enc
 }
 
 
+
+
 int stricomp(const char *s1,const char *s2)
 {
 #ifdef _WIN_ALL
@@ -117,46 +121,26 @@ wchar* RemoveLF(wchar *Str)
 }
 
 
-unsigned char loctolower(unsigned char ch)
-{
-#if defined(_WIN_ALL)
-  // Convert to LPARAM first to avoid a warning in 64 bit mode.
-  return (int)(LPARAM)CharLowerA((LPSTR)ch);
-#else
-  return tolower(ch);
-#endif
-}
-
-
-unsigned char loctoupper(unsigned char ch)
-{
-#if defined(_WIN_ALL)
-  // Convert to LPARAM first to avoid a warning in 64 bit mode.
-  return (int)(LPARAM)CharUpperA((LPSTR)ch);
-#else
-  return toupper(ch);
-#endif
-}
-
-
-// toupper with English only results if English input is provided.
-// It avoids Turkish (small i) -> (big I with dot) conversion problem.
-// We do not define 'ch' as 'int' to avoid necessity to cast all
+#if defined(SFX_MODULE)
+// char version of etoupperw. Used in console SFX module only.
+// Fast toupper for English only input and output. Additionally to speed,
+// it also avoids Turkish small i to big I with dot conversion problem.
+// We do not define 'c' as 'int' to avoid necessity to cast all
 // signed chars passed to this function to unsigned char.
-unsigned char etoupper(unsigned char ch)
+unsigned char etoupper(unsigned char c)
 {
-  if (ch=='i')
-    return 'I';
-  return toupper(ch);
+  return c>='a' && c<='z' ? c-'a'+'A' : c;
 }
+#endif
 
 
-// Unicode version of etoupper.
-wchar etoupperw(wchar ch)
+// Fast toupper for English only input and output. Additionally to speed,
+// it also avoids Turkish small i to big I with dot conversion problem.
+// We do not define 'c' as 'int' to avoid necessity to cast all
+// signed wchars passed to this function to unsigned char.
+wchar etoupperw(wchar c)
 {
-  if (ch=='i')
-    return 'I';
-  return toupperw(ch);
+  return c>='a' && c<='z' ? c-'a'+'A' : c;
 }
 
 
@@ -235,8 +219,8 @@ uint GetDigits(uint Number)
 
 bool LowAscii(const char *Str)
 {
-  for (int I=0;Str[I]!=0;I++)
-    if ((byte)Str[I]<32 || (byte)Str[I]>127)
+  for (size_t I=0;Str[I]!=0;I++)
+    if (/*(byte)Str[I]<32 || */(byte)Str[I]>127)
       return false;
   return true;
 }
@@ -244,11 +228,11 @@ bool LowAscii(const char *Str)
 
 bool LowAscii(const wchar *Str)
 {
-  for (int I=0;Str[I]!=0;I++)
+  for (size_t I=0;Str[I]!=0;I++)
   {
     // We convert wchar_t to uint just in case if some compiler
     // uses signed wchar_t.
-    if ((uint)Str[I]<32 || (uint)Str[I]>127)
+    if (/*(uint)Str[I]<32 || */(uint)Str[I]>127)
       return false;
   }
   return true;
@@ -275,53 +259,49 @@ int wcsnicompc(const wchar *s1,const wchar *s2,size_t n)
 }
 
 
-// Safe strncpy: copies maxlen-1 max and always returns zero terminated dest.
-char* strncpyz(char *dest, const char *src, size_t maxlen)
+// Safe copy: copies maxlen-1 max and for maxlen>0 returns zero terminated dest.
+void strncpyz(char *dest, const char *src, size_t maxlen)
 {
   if (maxlen>0)
   {
-    strncpy(dest,src,maxlen-1);
-    dest[maxlen-1]=0;
+    while (--maxlen>0 && *src!=0)
+      *dest++=*src++;
+    *dest=0;
   }
-  return dest;
 }
 
 
-// Safe wcsncpy: copies maxlen-1 max and always returns zero terminated dest.
-wchar* wcsncpyz(wchar *dest, const wchar *src, size_t maxlen)
+// Safe copy: copies maxlen-1 max and for maxlen>0 returns zero terminated dest.
+void wcsncpyz(wchar *dest, const wchar *src, size_t maxlen)
 {
   if (maxlen>0)
   {
-    wcsncpy(dest,src,maxlen-1);
-    dest[maxlen-1]=0;
+    while (--maxlen>0 && *src!=0)
+      *dest++=*src++;
+    *dest=0;
   }
-  return dest;
 }
 
 
-// Safe strncat: resulting dest length cannot exceed maxlen and dest 
-// is always zero terminated. Note that 'maxlen' parameter defines the entire
-// dest buffer size and is not compatible with standard strncat.
-char* strncatz(char* dest, const char* src, size_t maxlen)
+// Safe append: resulting dest length cannot exceed maxlen and dest 
+// is always zero terminated. 'maxlen' parameter defines the entire
+// dest buffer size and is not compatible with wcsncat.
+void strncatz(char* dest, const char* src, size_t maxlen)
 {
-  size_t Length = strlen(dest);
-  int avail=int(maxlen - Length - 1);
-  if (avail > 0)
-    strncat(dest, src, avail);
-  return dest;
+  size_t length = strlen(dest);
+  if (maxlen > length)
+    strncpyz(dest + length, src, maxlen - length);
 }
 
 
-// Safe wcsncat: resulting dest length cannot exceed maxlen and dest 
-// is always zero terminated. Note that 'maxlen' parameter defines the entire
-// dest buffer size and is not compatible with standard wcsncat.
-wchar* wcsncatz(wchar* dest, const wchar* src, size_t maxlen)
+// Safe append: resulting dest length cannot exceed maxlen and dest 
+// is always zero terminated. 'maxlen' parameter defines the entire
+// dest buffer size and is not compatible with wcsncat.
+void wcsncatz(wchar* dest, const wchar* src, size_t maxlen)
 {
-  size_t Length = wcslen(dest);
-  int avail=int(maxlen - Length - 1);
-  if (avail > 0)
-    wcsncat(dest, src, avail);
-  return dest;
+  size_t length = wcslen(dest);
+  if (maxlen > length)
+    wcsncpyz(dest + length, src, maxlen - length);
 }
 
 
@@ -428,7 +408,7 @@ const wchar* GetCmdParam(const wchar *CmdLine,wchar *Param,size_t MaxSize)
 }
 
 
-#ifndef SILENT
+#ifndef RARDLL
 // For compatibility with existing translations we use %s to print Unicode
 // strings in format strings and convert them to %ls here. %s could work
 // without such conversion in Windows, but not in Unix wprintf.
