@@ -8,20 +8,21 @@ void IntToExt(const char *Src,char *Dest,size_t DestSize);
 enum ACTW_ENCODING { ACTW_DEFAULT, ACTW_OEM, ACTW_UTF8};
 void ArcCharToWide(const char *Src,wchar *Dest,size_t DestSize,ACTW_ENCODING Encoding);
 
+
 int stricomp(const char *s1,const char *s2);
 int strnicomp(const char *s1,const char *s2,size_t n);
 wchar* RemoveEOL(wchar *Str);
 wchar* RemoveLF(wchar *Str);
-unsigned char loctolower(unsigned char ch);
-unsigned char loctoupper(unsigned char ch);
 
-char* strncpyz(char *dest, const char *src, size_t maxlen);
-wchar* wcsncpyz(wchar *dest, const wchar *src, size_t maxlen);
-char* strncatz(char* dest, const char* src, size_t maxlen);
-wchar* wcsncatz(wchar* dest, const wchar* src, size_t maxlen);
+void strncpyz(char *dest, const char *src, size_t maxlen);
+void wcsncpyz(wchar *dest, const wchar *src, size_t maxlen);
+void strncatz(char* dest, const char* src, size_t maxlen);
+void wcsncatz(wchar* dest, const wchar* src, size_t maxlen);
 
-unsigned char etoupper(unsigned char ch);
-wchar etoupperw(wchar ch);
+#if defined(SFX_MODULE)
+unsigned char etoupper(unsigned char c);
+#endif
+wchar etoupperw(wchar c);
 
 bool IsDigit(int ch);
 bool IsSpace(int ch);
@@ -43,7 +44,7 @@ void itoa(int64 n,char *Str,size_t MaxSize);
 void itoa(int64 n,wchar *Str,size_t MaxSize);
 const wchar* GetWide(const char *Src);
 const wchar* GetCmdParam(const wchar *CmdLine,wchar *Param,size_t MaxSize);
-#ifndef SILENT
+#ifndef RARDLL
 void PrintfPrepareFmt(const wchar *Org,wchar *Cvt,size_t MaxSize);
 #endif
 
